(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: configuration.ml,v 1.47 2013-12-02 15:28:38 fclement Exp $ *)

module Nat = Unatural;;

let c_invalid_arg = Upervasives.invalid_arg "Configuration";;

open Segmentation_types;;
open Configuration_types;;
open Configuration_types_print;;
open Options;;

(*let gen_digest = Utype_digest.gen_digest Configuration_digest.md5;;
let digest = gen_digest "configuration";;*)

(* History functor interface. *)

let print = print_t;;

let get_dimension () = Nat.of_int (get_option_dimension ());;

let get_dimension_management () =
  match get_option_dimension_management () with
  | Option_Vector -> (Best_cutting, All_components)
  | Option_Best_component_only -> (Best_cutting, Best_component)
  | Option_Best_component_for_each -> (All_cuttings, Best_component)
  | Option_Combine_best_components -> (All_cuttings, All_components)
;;

let bcf_of_option = function
  | Option_Horizontal -> Horizontal
  | Option_Vertical -> Vertical
;;

let get_cutting_strategy () =
  match get_option_cutting_strategy () with
  | Option_Best_in_a_family ->
      let nct = Nat.of_int (get_option_number_of_cuts ()) in
      let bcfl =
        Ulist.map bcf_of_option (get_option_basic_cutting_families ()) in
      Best_in_a_family (nct, Ulist1.of_list bcfl)
  | Option_Overall_best -> Overall_best
;;

let get_stopping_criterion () =
  {
   max_iteration = Nat.of_int (get_option_max_iteration ());
   min_best_scalar_indicator = get_option_min_best_scalar_indicator ();
   min_scalar_relative_error = get_option_min_scalar_relative_error ();
  }
;;

let get_image_width = get_argument_initial_image_width;;
let get_image_height = get_argument_initial_image_height;;

let { Uaref.get = get_configuration_opt; set = set_configuration_opt; } =
  Uaref.make_aref_unique "algorithm configuration" None
;;

let cold_set () =
  let config =
    {
     dimension = get_dimension ();
     dimension_management = get_dimension_management ();
     cutting_strategy = get_cutting_strategy ();
     stopping_criterion = get_stopping_criterion ();
     image_width = get_image_width ();
     image_height = get_image_height ();
   } in
  set_configuration_opt (Some config)
;;

let warm_set config =
  let config =
    { config with stopping_criterion = get_stopping_criterion (); } in
  set_configuration_opt (Some config)
;;

let get () =
  match get_configuration_opt () with
  | None -> assert false
  | Some config -> config
;;

(* Algorithm configuration. *)

let phase_dimension = Options.phase_dimension;;

let get_dimension () = (get ()).dimension;;
let get_dimension_management () = (get ()).dimension_management;;
let get_cutting_strategy () = (get ()).cutting_strategy;;
let get_stopping_criterion () = (get ()).stopping_criterion;;
let get_image_width () = (get ()).image_width;;
let get_image_height () = (get ()).image_height;;

(* Program configuration. *)

let get_formatter () =
  match get_option_out_formatter () with
  | Option_Std_out -> Format.std_formatter
  | Option_Std_err -> Format.err_formatter
  | Option_Out_file fname ->
      Format.formatter_of_out_channel (Pervasives.open_out fname)
;;

let get_report_allocated_memory = get_option_report_allocated_memory;;

let get_incremental_mode = get_option_incremental_mode;;

let get_graphics_mode () =
  match get_option_graphics_mode () with
  | Option_No_graphics ->
      { on = false; kind = None; }
  | Option_Minimal_graphics ->
      { on = true; kind = Some (Param_quantity, Compound_channel); }
  | Option_Parameter_graphics ->
      { on = true; kind = Some (Param_quantity, All_channels); }
  | Option_Compound_graphics ->
      { on = true; kind = Some (All_quantities, Compound_channel); }
  | Option_Full_graphics ->
      { on = true; kind = Some (All_quantities, All_channels); }
;;

let check_graphics_window_sizes () =
  let w_min = get_option_graphics_window_min_width ()
  and h_min = get_option_graphics_window_min_height () in
  let w_max = get_option_graphics_window_max_width ()
  and h_max = get_option_graphics_window_max_height () in
  if w_max < w_min then
    c_invalid_arg "" "Min width is greater than max width" else
  if h_max < h_min then
    c_invalid_arg "" "Min height is greater then max height"
;;

let get_graphics_window_min_size () =
  check_graphics_window_sizes ();
  let w_min = get_option_graphics_window_min_width ()
  and h_min = get_option_graphics_window_min_height () in
  (Nat.of_int w_min, Nat.of_int h_min)
;;

let get_graphics_window_max_size () =
  check_graphics_window_sizes ();
  let w_max = get_option_graphics_window_max_width ()
  and h_max = get_option_graphics_window_max_height () in
  (Nat.of_int w_max, Nat.of_int h_max)
;;

let get_graphics_gradient_threshold () =
  Nat.of_int (get_option_graphics_gradient_threshold ())
;;

let get_final_image_file_name () = get_option_final_image_file_name ();;

(* Monitoring. *)

let {
  Udebug.report = report;
  report_action = report_action;
  debug = debug;
  debug_action = debug_action;
  fatal_error = fatal_error;
} = Options.debug_printing_facilities
;;

let print_ident_value ident printer ppf value =
  Format.fprintf ppf "%s =@ %a" ident printer value
;;

let print_report printer ppf value =
  report_action
    (fun () -> Format.fprintf ppf "@[<hv>%a@]" printer value; "")
;;

let print_debug fun_name ident printer ppf value =
  debug_action fun_name
    (fun () ->
      Format.fprintf ppf "@[<v 2>%s: %a@]@."
        fun_name (print_ident_value ident printer) value; "")
;;

let print_debug_no_arg fun_name printer ppf =
  debug_action fun_name
    (fun () -> Format.fprintf ppf "@[<v 2>%s: %t@]@." fun_name printer; "")
;;
