(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: upervasives.mli,v 1.31 2014/01/10 15:28:04 fclement Exp $ *)

(** {3 Extensions to the initially opened module} *)

(** {6 Error messages} *)

type module_name = string
and function_name = string
and message = string
;;

val error_message : module_name -> function_name -> message -> message;;

(** {6 Exceptions} *)

val invalid_arg : module_name -> function_name -> message -> 'a;;
val failwith : module_name -> function_name -> message -> 'a;;

(** {6 Functional helpers} *)

external id : 'a -> 'a = "%identity";;
(** [id x] returns [x], it is the identity function. *)

val compose : ('b -> 'c) -> ('a -> 'b) -> 'a -> 'c;;

val proj2_1 : 'a -> 'b -> 'a;;
val proj2_2 : 'a -> 'b -> 'b;;

val option : ('a -> 'b) -> 'b -> 'a option -> 'b;;
(** [option f default opt] will either apply [f] to the value in [opt] if
   this is [Some _], otherwise it will return [default]. *)

val lifto : ('a -> 'b) -> 'a option -> 'b option;;
(** [lifto f] lifts [f] to the type option. If the input is [None],
   it returns [None], otherwise it applies [f] to the value enclosed in the
   option, and returns the corresponding option. *)

(** {6 Integer arithmetic} *)

type index = Unatural.t;;

val index_of_int : int -> index;;
val index_to_int : index -> int;;

type length = Unatural.t;;

val length_of_int : int -> length;;
val length_to_int : length -> int;;

(** {6 Floating-point arithmetic} *)

val round : float -> int;;
(** Round the given float to the nearest integer.
  For a nonnegative float [f], [round f] returns Pervasives.int_of_float
  [(f +. 0.5)], ie the greatest integer less than or equal to [f +. 0.5];
  and for a negative float [f], [round f] returns Pervasives.int_of_float
  [(f -. 0.5)], ie the least integer greater than or equal to [f -. 0.5]. *)

(** {6 Conversion} *)

val bit_string_of_float : float -> string;;
(** [bit_string_of_float f] returns a [string] containing the string of bits
  of the binary representation of the float [f]. *)

val float_of_bit_string : string -> float;;
(** [float_of_bit_string] is the converse function of [bit_string_of_float]. *)

(** {6 Input/output} *)

val print_unit : unit -> unit;;
(** Print unit on standard output. *)

val prerr_unit : unit -> unit;;
(** Print unit on standard error. *)

val print_bool : bool -> unit;;
(** Print a boolean on standard output. *)

val prerr_bool : bool -> unit;;
(** Print a boolean on standard error. *)

val open_append : string -> out_channel;;
val open_append_bin : string -> out_channel;;
