(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uuniverse.mli,v 1.5 2014/01/10 15:28:04 fclement Exp $*)

(** {3 Universes} *)

type 'a index = 'a Uindex.t;;

type 'a indices;;
type 'a elements = 'a array;;

type 'a t = private
  {
   indices : 'a indices;
   mutable elements : 'a elements;
   mutable cardinal : Unatural.t;
  }
;;
(** A universe is the support of a family of sets. It is a set that
  contains all the elements that belong to a set of the family of sets.
  In short, any set we consider in this library is a subset of some universe.
*)

(** {6 Expansion phase, or transient state} *)

val make : Unatural.t -> 'a -> 'a t;;

val add : 'a t -> 'a -> unit;;
(*
val add_list : 'a t -> 'a Ulist.t -> unit;;
val add_array : 'a t -> 'a Uarray.t -> unit;;
*)

val copy : 'a t -> 'a t;;

val product : 'a t -> 'b t -> ('a * 'b) t;;

val of_list : 'a Ulist1.t -> 'a t;;
(** [ Uuniverse.of_list [ x1; x2; ...; xn ] ] creates the universe
  made of all distinct elements of the list [ x1; x2; ...; xn ]. *)

val of_array : 'a array -> 'a t;;
(** [Uuniverse.of_array a] is equivalent to
  [Uuniverse.of_list (Array.to_list a)]. *)

val of_range : int -> int -> int t;;
(** [Uuniverse.of_range i j] Create the universe for integer in the
  range  [\[ i, j \[].
  [Uuniverse.of_range i j] is equivalent to
  [Uuniverse.of_list (i, \[ i + 1; ... j - 1 \])].
  Raise exception [Invalid_arg "of_range"]
  if [i] is greater or equal to [j]. *)

val of_natural_range : Unatural.t -> Unatural.t -> Unatural.t t;;
(** [of_natural_range i j] Create the universe for natural numbers in the
  range  [\[ i, j \[].
  [Uuniverse.of_range i j] is equivalent to
  [Uuniverse.of_list (i, \[ i + 1; ... j - 1 \])].
  Raise exception [Invalid_arg "of_range"]
  if [i] is greater or equal to [j]. *)

val of_rectangle : int -> int -> (int * int) t;;

val of_natural_rectangle :
  Unatural.t -> Unatural.t -> (Unatural.t * Unatural.t) t
;;

(** {6 Steady state} *)

val stats : 'a t -> Hashtbl.statistics;;

val cardinal : 'a t -> Unatural.t;;

val valid_index : 'a t -> 'a index -> bool;;
val check_index :
  Upervasives.module_name -> Upervasives.function_name ->
  'a t -> 'a index -> unit
;;

val index : 'a t -> 'a -> 'a index;;
val indices : 'a t -> 'a index t;;

val element : 'a t -> 'a index -> 'a;;
val elements : 'a t -> 'a array;;
val sub_elements : 'a t -> 'a index array -> 'a array;;

val mem : 'a t -> 'a -> bool;;

val fold_left : ('accu -> 'a -> 'accu) -> 'accu -> 'a t -> 'accu;;
val fold_left_i :
  ('accu -> 'a index -> 'a -> 'accu) -> 'accu -> 'a t -> 'accu
;;

val iter : ('a -> unit) -> 'a t -> unit;;
val iter_i : ('a index -> 'a -> unit) -> 'a t -> unit;;

val map_elements : ('a -> 'b) -> 'a t -> 'b array;;

val map : ('a -> 'b) -> 'a t -> 'b t;;

val find : ('a -> bool) -> 'a t -> 'a;;

val find_all : ('a -> bool) -> 'a t -> 'a list;;

val to_list : 'a t -> 'a list;;
